+{
   locale_version => 1.31,
   entry => <<'ENTRY', # for DUCET v13.0.0
0D82      ; [.2C61.0020.0002][.FFF1.0000.0000] # SINHALA SIGN ANUSVARAYA
0D83      ; [.2C61.0020.0002][.FFF2.0000.0000] # SINHALA SIGN VISARGAYA
0D96      ; [.2C61.0020.0002][.FFF0.0000.0000] # SINHALA LETTER AUYANNA
0DA5      ; [.2C6C.0020.0002] # SINHALA LETTER TAALUJA SANYOOGA NAAKSIKYAYA
0DA4      ; [.2C6D.0020.0002] # SINHALA LETTER TAALUJA NAASIKYAYA
ENTRY
};
